﻿using System;

using System.Collections.Generic;

using System.Text;

using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Text;
using System.Drawing.Printing;

namespace cristian_docentes
{
    /// <summary>
    /// class printer
    /// 
    /// Gestiona la impresión de los registros de la base de datos.
    /// 
    /// Detecta el número de impresoras disponibles en el equipo y, en caso de ser más de una pregunta
    /// al usuario si desea seleccionar una diferente a la establecida por defecto.
    /// 
    /// </summary>
    public class printer
    {
        /* Declaramos los objetos que intervienen en la impresión del documento */
        PrintDocument               doc;
        PageSettings                pageSets;
        PrinterSettings             printerSets;
        StandardPrintController     controller;
        Font printerFont;
        /* Variables que utilizaremos en el control posicional de la impresora */
        float                       leftPos         = 0;
        float                       topPos          = 0;
        float                       linesPerPage    = 0;

        /// <summary>
        /// Clase encargada de enviar a la impresora por defecto del sistema
        /// el contenido de los registros de la base de datos.
        /// </summary>
        public printer()
        {
            /* Si no se dispone de una impresora instalada el documento no se puede procesar */
            if (PrinterSettings.InstalledPrinters.Count >= 1)
            {
                /* Crear instancia de los objetos que intervienen en la impresión del documento */
                doc                     = new PrintDocument();
                pageSets                = doc.DefaultPageSettings;
                printerSets             = doc.PrinterSettings;
                controller              = new StandardPrintController();
                doc.DocumentName        = "Docentes DB";
                printerFont             = new Font("Courier New", 10);

                /* Capturamos el evento que detecta la impresión del documento */
                doc.PrintPage           += Doc_PrintPage;

                /* Solicitamos si el usuario desea modificar la impresora por defecto del sistema si el sistema tiene más de una disponible */
                if (PrinterSettings.InstalledPrinters.Count > 1)
                {
                    if (MessageBox.Show("La impresora por defecto es: " + getDefaultPrinter() + "\r\n\r\nDesea seleccionar otra impresora para este documento ?", "IMPRESORA", MessageBoxButtons.YesNo) == DialogResult.Yes)
                    {
                        selectPrinter();
                    }
                }

                /* Lanza proceso de impresión en evento PrintPage (Ver: Doc_PrintPage) */
                doc.Print();
            }
            else
            {
                MessageBox.Show("No se detectan impresoras instaladas en el sistema.\r\nNo es posible crear documento solicitado.", "PRINTERS", MessageBoxButtons.OK);
            }
        }

        /* Evento que controla el proceso de impresión */
        private void Doc_PrintPage(object sender, PrintPageEventArgs ev)
        {
            Console.Clear();
            Console.WriteLine("Impresión en curso.");

            /* Iniciamos valores de las propiedades de la página */
            leftPos         = ev.MarginBounds.Left;
            topPos          = ev.MarginBounds.Top;
            linesPerPage    = ev.MarginBounds.Height / printerFont.GetHeight(ev.Graphics);

            float[] x       = new float[3];
            int count       = 0;

            /* Calculamos las posiciones de los datos en el papel */
            x[0]            = ev.Graphics.MeasureString("99999", printerFont).Width - ev.Graphics.MeasureString("99999", printerFont).Width;
            x[1]            = x[0] + ev.Graphics.MeasureString("AAAAAAAAA", printerFont).Width;
            x[2]            = x[1] + ev.Graphics.MeasureString("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA", printerFont).Width;


            /* Recorremos e imprimimos la colección de datos existentes en memoria */
            if (Program.db.registros.Count == 0)
            {
                MessageBox.Show("La base de datos no idpsone de ningún registro\r\nNo puede generarse documento de impresión.", "DATA_BASE", MessageBoxButtons.OK);
            }
            else
            {
                foreach (KeyValuePair<string, docentes> d in Program.db.registros)
                {
                    /* Imprimir encabezado si es la primera línea de cada página (count = 0) */
                    if (count == 0)
                    {
                        topPos = 0;

                        /* Ident */
                        ev.Graphics.DrawString("Ident", printerFont, Brushes.Black, x[0], topPos);
                        /* Nombre */
                        ev.Graphics.DrawString("Nombre", printerFont, Brushes.Black, x[1], topPos);
                        /* Titulación */
                        ev.Graphics.DrawString("Titulación", printerFont, Brushes.Black, x[2], topPos);

                        count++;
                        topPos = ((float)count * printerFont.GetHeight(ev.Graphics));

                        ev.Graphics.DrawString(d.Value.ident.ToString("====="), printerFont, Brushes.Black, x[0], topPos);
                        ev.Graphics.DrawString(d.Value.ident.ToString("=================================================="), printerFont, Brushes.Black, x[1], topPos);
                        ev.Graphics.DrawString(d.Value.ident.ToString("==============="), printerFont, Brushes.Black, x[2], topPos);

                        count++;
                        topPos = ((float)count * printerFont.GetHeight(ev.Graphics));
                    }

                    ev.Graphics.DrawString(d.Value.ident.ToString("#,###0"), printerFont, Brushes.Black, x[0], topPos);
                    ev.Graphics.DrawString(d.Value.nombre.ToString(), printerFont, Brushes.Black, x[1], topPos);
                    ev.Graphics.DrawString(d.Value.categoria.ToString(), printerFont, Brushes.Black, x[2], topPos);

                    /* Incrementamos el número de línea de la página */
                    count++;
                    if (count >= linesPerPage)
                    {
                        /* Si hemos llenado las líneas totales de página iniciamos el contador para saltar de página e imprimir un nuevo encabezado */
                        count = 0;
                    }
                    else
                    {
                        /* Calculamos la posición en el papel de la siguiente línea de impresión */
                        topPos = ((float)count * printerFont.GetHeight(ev.Graphics));
                    }

                }
                /* Cerramos el documento, Fin de Impresión */
                doc.Dispose();
            }
            Console.Clear();
        }

        /// <summary>
        /// Función que enumera y selecciona la impresora por defecto para este documento
        /// </summary>
        private void selectPrinter()
        {
            ConsoleKeyInfo key;

            /* Mostrar impresoras */
            Console.Clear();
            Console.WriteLine("Impresoras disponibles");
            Console.WriteLine("======================================================================");
            /* Guardamos en una lista de tipo string los nombres de impresoras instaladas */
            List<string> printers = new List<string>();
            for (int i = 0; i < PrinterSettings.InstalledPrinters.Count; i++)
            {
                string prnt = (i + 1).ToString() + ".-  " + PrinterSettings.InstalledPrinters[i];
                printers.Add(PrinterSettings.InstalledPrinters[i]);

                Console.Write((i + 1).ToString() + ".-  ") ;
                Console.WriteLine (PrinterSettings.InstalledPrinters[i]);
            }
            Console.WriteLine();
            Console.WriteLine("Seleccione número de impresora o pulse -Escape- para mantener la instalada por defecto en el sistema.");

            /* Menú selector de impresora */
            bool select = true;
            do
            {
                key = Console.ReadKey(true);
                if (key.Key == ConsoleKey.Escape)
                {
                    select = false;
                }
                else
                {
                    char tecla = key.KeyChar;
                    try
                    {
                        char opt = (char)(tecla);
                        int valorTecla = int.Parse(opt.ToString());
                        if (valorTecla >= 0 && (valorTecla - 1) <= printers.Count)
                        {
                            doc.PrinterSettings.PrinterName = printers[valorTecla - 1];
                        }
                    }
                    catch (Exception ex)
                    {
                        string error = ex.Message;
                        MessageBox.Show("Error producido en el intento de asignar impresora por defecto para este documento.\r\n" + error, "PRINTERS", MessageBoxButtons.OK);
                    }
                    select = false;
                }
            } while (select == true);
        }

        /// <summary>
        /// Devuelve el nombre de la impresora por defecto del sistema
        /// </summary>
        /// <returns></returns>
        private string getDefaultPrinter()
        {
            string defprinter = doc.PrinterSettings.PrinterName; ;
            return defprinter;
        }
    }
}
